SR.entered_file()
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
--                            REMOVE SHADOW ANIMATIONS                            --
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

local shadow_anims = {
  -- combat-robot, construction-robot, logistic-robot
  "shadow_idle",
  "shadow_in_motion",

  -- construction-robot
  "shadow_working",

  -- logistic-robot
  "shadow_idle_with_cargo",
  "shadow_in_motion_with_cargo",

  -- "Companion Drones": spider-vehicle
  "shadow_base_animation",
  "shadow_animation",
}

-- Width and height of the empty image. Make this a variable so we can replace the
-- image with a bigger one if there should ever be a mod that will use more than
-- 64 directions!
local empty_image_size = 8


------------------------------------------------------------------------------------
--                                    Functions                                   --
------------------------------------------------------------------------------------
-- We use an empty image measuring 8x8 pixels. That's enough for a direction_count
-- up to 64 in the animation without shadows.
local function make_empty_shadow(pics, repeat_count)
  SR.entered_function({pics, repeat_count})

  -- How many lines do we need to represent all pictures in the grid?
  -- (Range: 1 … empty_image_size)
  local lines       = math.ceil(pics / empty_image_size)
SR.show("lines", lines)

  -- How many pictures can we fit into one line? (Range: 1 … empty_image_size)
  local per_line    = pics / lines
SR.show("per_line", per_line)

  local anim = {
    direction_count = pics,
    draw_as_shadow = true,
    filename = "__SmallRobots__/empty.png",
    frame_count = 1,
    height = empty_image_size / lines,
    line_length = per_line,
    priority = "very-low",
    width = empty_image_size / per_line,
    repeat_count = repeat_count
  }
  anim.hr_version = table.deepcopy(anim)
SR.show("anim", anim)

  SR.entered_function("leave")
  return anim
end




local function remove_shadows(bot)
  local anim_root, no_shadow, directions, repeat_count

  for a, animation in pairs(shadow_anims) do
    anim_root = (bot.type == "spider-vehicle") and bot.graphics_set or bot
SR.show("Checking animation \""..animation.."\"", anim_root[animation])
    if anim_root[animation] then
      -- Shadow animation and no-shadow animation must have the same direction and
      -- frame count. If the no-shadow animation has more frames than the shadow
      -- animation, repeat_count can be used to keep both versions in sync.
      no_shadow = animation:match("shadow_(.+)")
SR.show("no_shadow", no_shadow)
SR.writeDebug("anim_root[%s]: %s", {SR.enquote(no_shadow), anim_root[no_shadow]})

      if anim_root[no_shadow] then
        directions = anim_root[no_shadow].direction_count or
                      anim_root[animation].direction_count
        repeat_count = anim_root[no_shadow].frame_count
      else
        directions = anim_root[animation].direction_count
        repeat_count = anim_root[animation].frame_count
      end

      -- Modify animation
      anim_root[animation] = make_empty_shadow(directions, repeat_count)
      SR.writeDebug("Created empty animation %s.", {SR.enquote(animation)})
    end
  end

    SR.entered_function("leave")
end



------------------------------------------------------------------------------------
--                                 Call functions                                 --
------------------------------------------------------------------------------------
SR.show("SR.scaled_prototypes", SR.scaled_prototypes)
for bot_type, bot_names in pairs(SR.scaled_prototypes) do
  SR.writeDebug("Checking bot type %s!", {SR.argprint(bot_type)})
  for b, bot_name in pairs(bot_names) do
    robot = data.raw[bot_type][bot_name]
    if robot then
      SR.writeDebug("Removing shadows from %s!", {SR.argprint(robot)})
      remove_shadows(robot)
    end
  end
end



------------------------------------------------------------------------------------
SR.entered_file("leave")
